package unlh.graph.datastructure;

import org.graphstream.graph.Graph;
import org.graphstream.graph.GraphFactory;
import org.graphstream.graph.Node;
import org.graphstream.graph.NodeFactory;
import org.graphstream.graph.implementations.SingleGraph;
import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.*;

class PrioritaryLineTest {

    PrioritaryLine prioritaryLine;
    Graph graph;
    NodeFactory nodeFactory;

    @BeforeEach
    void setUp() {
        graph = new SingleGraph("testGraph");
        nodeFactory = new NodeFactory() {
            @Override
            public Node newInstance(String id, Graph graph) {
                return graph.addNode(id);
            }
        };
        prioritaryLine = new PrioritaryLine();
    }

    @AfterEach
    void tearDown() {
        prioritaryLine = null;
    }

    @Test
    void init() {
        assertNull(prioritaryLine.map, "L'attribut map est null");
    }

    @Test
    void empty() throws Exception {
        prioritaryLine.init();
        assertTrue(prioritaryLine.empty());
    }

    @Test
    void clear() {
        Node node = nodeFactory.newInstance("1", graph);
        prioritaryLine.init();
        prioritaryLine.add(node, Double.valueOf(1.0));
        assertNotNull(prioritaryLine, "Calling clear method on non initialized Prioritary line");
        assertTrue(prioritaryLine.length() == 1, "Length is not equals to one");
        prioritaryLine.clear();
        assertTrue(prioritaryLine.length() ==0, "Length is not equals to zero");
    }

    @Test
    void length() {
        prioritaryLine.init();
        Node node = nodeFactory.newInstance("1", graph);
        assertNotNull(prioritaryLine.map, "Calling clear method on non initialized Prioritary line");
        assertEquals(prioritaryLine.length(), 0, "Lengths are not equal to zero");
        prioritaryLine.add(node, Double.valueOf(2));
        assertTrue(prioritaryLine.length() == 1, "Length not equal to 1 on add");
    }

    @Test
    void extractMin() {
        prioritaryLine.init();
        Node node = (Node) nodeFactory.newInstance("1", graph);
        prioritaryLine.add(node, Double.valueOf(2));
        assertNotNull(prioritaryLine.extractMin(), "Calling extractMin on non initialized or empty map attribute");
        assertTrue(prioritaryLine.length() > 0, "Length inferior or equals to zero");
        assertEquals(prioritaryLine.extractMin(), node, "Min value not equals");
    }
}